import 'dart:typed_data';

import 'type_adapter.dart';
import 'type_registry.dart';

/// The [BinaryReader] is used to bring data back from the binary format.
abstract class BinaryReader {
  const BinaryReader(this.registry);

  final TypeRegistry registry;

  /// The number of bytes left in this entry.
  int get availableBytes;

  /// The number of read bytes.
  int get usedBytes;

  /// Skip n bytes.
  void skip(int bytes);

  /// Get a [Uint8List] view which contains the next [bytes] bytes.
  Uint8List readBytes(int bytes);

  /// Get a [Uint8List] view which contains the next [bytes] bytes. This does
  /// not advance the internal read position.
  Uint8List peekBytes(int bytes);

  T read<T>(TypeAdapter<T> adapter) => adapter.read(this);
}
